///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2012 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//      TF  Jan 03, 2012    Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import java.io.File;
import java.io.IOException;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.data.TdbFieldTemplate;
import com.tietoenator.trip.jxp.data.TdbPhraseField;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbRendition;
import com.tietoenator.trip.jxp.data.TdbHighlightRendition;
import com.tietoenator.trip.jxp.data.TdbRenditionType;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.data.TdbStringField;
import com.tietoenator.trip.jxp.data.TdbTextField;
import com.tietoenator.trip.jxp.session.TdbSession;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;


/**
 * This class demonstrates how to use TRIPcof HTML conversion on the server
 * and locally. For server-side use, TRIPcof must be installed together with
 * TRIPsystem. For client-side use, TRIPcof must be installed on the machine
 * on which this example program is being run.
 *
 * <p>Use the TRIPcof sample database with this example program. Database
 * definition files for it are available in the TRIPcof installation.</p>
 *
 * <p><b>NOTE:</b>
 * To run this program with client-side processing, you must first define the
 * Java system property or environment variable TRIPCOF_HOME to be the
 * fully qualified path to the installation directory of TRIPcof on the local
 * machine.
 * </p>
 */
public class CofConvert
{
    private TdbSession m_session;
    private String m_database = null;
    private String m_stringField = "FILE_BLOB";
    private String m_fileNameField = "FILE_NAME";

    /**
     * Constructor
     * @throws TdbException
     */
    public CofConvert()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIPcof HTML Conversion Example ****", false);
    }

    void close()
    {
        if (m_session != null)
        {
            try { m_session.logout(); } catch (Exception ignore) {}
            m_session = null;
        }
    }

    private void setDatabase(String db)
    {
        m_database = db;
    }

    private void setStringField(String sf)
    {
        m_stringField = sf;
    }

    private void setFileNameField(String nf)
    {
        m_fileNameField = nf;
    }

    public void process() throws TdbException, IOException
    {
        boolean clientside = false;
        TdbFieldTemplate fld = null;
        TdbRenditionType rtype = null;

        TdbRecord rec = null;
        TdbSearch s = null;
        String textFieldName = "FILE_TEXT";
        boolean refreshHitLocations = true;
        boolean highlighting = false;
        String query = "";
        int recnr;
        
        Console stdin = new Console();

        int rend = stdin.readint("Rendition (0-none, 1-HTML, 2-MHT)", 0, true);
        if (rend > 0)
        {
            query = stdin.readln("Query condition", "", true);
            if (query.length() > 0)
            {
                recnr = stdin.readint("Record ordinal (RIS) #", 1, true);
                if (0 != stdin.readint("Apply highlighting (0=no, 1=yes)", 0, true))
                    highlighting = true;

                if (highlighting)
                    textFieldName = stdin.readln("Text field with extracted content", "FILE_TEXT", true);
            }
            else
                recnr = stdin.readint("Record ID #", 1, true);
        }
        else
            recnr = stdin.readint("Record ID #", 1, true);

        if (0 != stdin.readint("Client-side Conversion (0=no, 1=yes)", 0, true))
            clientside = true;

        // Set appropriate rendition type according to user request
        if (rend == 1)
        {
           rtype = TdbRenditionType.BasicHTML;
        }
        else if (rend == 2)
        {
           rtype = TdbRenditionType.MimeEncodedHtml;
        }

        if (rend > 0 && !clientside)
        {
            if (highlighting)
                fld = new TdbHighlightRendition(m_stringField, textFieldName, m_fileNameField, rtype);
            else
                fld = new TdbRendition(m_stringField, m_fileNameField, rtype);
        }
        else if (rend < 1 || clientside)
        {
           fld = new TdbFieldTemplate(m_stringField);
        }

        String outputFile = null;
        while (outputFile == null)
        {
            outputFile = stdin.readln("Output file name", null);
        }

        System.out.println();

        // Split output file into directory and file as required by
        // the local conversion
        String outputDir = "";
        if (clientside)
        {
            File fOutput = new File(outputFile);
            fOutput = new File(fOutput.getAbsolutePath());
            outputDir = fOutput.getParentFile().getAbsolutePath();
            outputFile = fOutput.getName();
        }

        // Create retrieval template
        TdbRecord tmpl = new TdbRecord(m_session,m_database,false);
        if (clientside)
        {
            tmpl.addToTemplate(m_fileNameField);
            tmpl.addToTemplate(m_stringField);
            if (highlighting)
                tmpl.addToTemplate(textFieldName);
        }
        else
            tmpl.addToTemplate(fld);

        System.out.println("... retrieving record ... ");

        if (query.length() > 0)
        {
            // Use TdbSearch - TdbRecordSet is not recommended with TRIPcof
            s = new TdbSearch(m_session);

            // Assign custom retrieval template
            s.setRetrievalTemplate(tmpl);
            s.setAutomaticRetrievalTemplate(false);

            // Execute CCL statements
            s.execute("DEFINE FIND MIN=0");
            s.execute("BASE " + m_database);
            s.execute("FIND " + query);

            TdbSearchSet ss = s.getLastSearchSet();
            if (ss.getRecordCount() < recnr)
            {
                System.out.println("ERROR: Only " + Integer.toString(ss.getRecordCount()) + " records found using specified query.");
                System.out.println("       Unable to convert record with RIS "+ Integer.toString(recnr) + ".");
                ss = null;
                s.close();
                return;
            }

            rec = ss.getRecord(recnr - 1);
        }
        else
        {
             rec = tmpl;
             rec.setRecordId(recnr);
             rec.get();
        }

        TdbStringField str = (TdbStringField) rec.getHead().getField(m_stringField);
        TdbPhraseField name = (TdbPhraseField) rec.getHead().getField(m_fileNameField);
        TdbTextField textfield = (TdbTextField) rec.getHead().getField(textFieldName);

        // We MUST have a file name in order for TRIPcof to work well.
        String storedname = "noname.doc";
        if (name != null && name.getValueCount() > 0)
            storedname = name.getValue(0);

        System.out.println("... retrieving rendition ... ");

        if (rend>0 && clientside)
        {
            // Client-side HTML conversion is done now.
            if (highlighting)
                str.convert(rtype, storedname, outputDir, outputFile, null, textfield, refreshHitLocations);
            else
                str.convert(rtype, storedname, outputDir, outputFile, null, null, false);
        }
        else
        {
            // For server-side conversion, we just output the retrieved
            // rendering.
            str.copyToFile(outputFile);
        }

        System.out.println();
        System.out.println("Document successfully converted.");
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        CofConvert program = null;
        try
        {
            program = new CofConvert();
            String db = null, blobfield = null, namefield = null;
            Console stdin = new Console();

            // Get a valid database name, and set the list of
            // retrieval fields from it
            while (db == null)
                db = stdin.readln("TRIPcof Sample Database", null);

            blobfield = stdin.readln("String field with file data", "FILE_BLOB", true);
            namefield = stdin.readln("Phrase field with file name", "FILE_NAME", true);

            program.setDatabase(db);
            program.setStringField(blobfield);
            program.setFileNameField(namefield);

            // Perform the operation
            program.process();
        }
        catch (TdbException e)
        {
            System.out.println();
            System.out.println("[ERROR]");
            System.out.println(e.getDebugMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (program != null)
            {
                program.close();
                program = null;
            }
        }
    }
}
