///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2010 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Feb 26, 2010	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.data.TdbField;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.data.TdbStructuredField;
import com.tietoenator.trip.jxp.data.TdbTextField;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.util.Vector;

/**
 * Sample program for TRIPjxp that shows how to use the TdbSearch class
 * to conduct a search and iterate through the results.
 */
public class SearchSetIteration
{
    private TdbSession m_session;
    private TdbDatabaseDesign m_db;

    /**
     * Constructor
     * @throws TdbException
     */
    public SearchSetIteration()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TdbSearchSet record iteration example ****", false);
    }

    public void loadDatabaseDesign(String database) throws TdbException
    {
        m_db = new TdbDatabaseDesign(m_session);
        m_db.get(database);
    }

    public boolean isDatabaseHeadField(String fieldname)
    {
        return (m_db.getFieldByName(fieldname) == null) ? false : true;
    }


    public void search(String qry, String refinement, Vector<String> fetchFields, int maxRecords)
            throws TdbException
    {
        int recordIdx = 0;
        TdbRecord template = null;

        // Create our search handler.
        TdbSearch searchHandler = new TdbSearch(m_session);

        // Use automatic retrieval templates if no fields have been specified.
        if (fetchFields.size() == 0)
            searchHandler.setAutomaticRetrievalTemplate(true);
        else
        {
            template = new TdbRecord(m_session,m_db,false);
            for (String fld : fetchFields)
                template.addToTemplate(fld);

            searchHandler.setAutomaticRetrievalTemplate(false);
            searchHandler.setRetrievalTemplate(template);
        }

        // Open the database
        searchHandler.execute("BASE " + m_db.getName());

        // Clear search history associated with this search handler.
        // Here it just removes the search set associated with the
        // BASE command.
        searchHandler.clear();

        // Execute the first search.
        searchHandler.execute("FIND " + qry);

        // Refine our search!
        if (refinement != null)
        {
            String strSearch = "FIND S=" +
                    Integer.toString(searchHandler.getLastSearchSet().getSearchId()) +
                    " AND ( " + refinement + " )";
            searchHandler.execute(strSearch);
        }

        TdbSearchSet searchSet = searchHandler.getLastSearchSet();
        /* if (template != null)
            searchSet.setRetrievalTemplate(template); */

        // Iterate through the found records.
        for (TdbRecord rec : searchSet)
        {
            System.out.print("\nRecord #");
            System.out.println(Integer.toString(rec.getRecordId()));

            // Display only head record fields
            for (TdbField f : rec.getHead().fields())
            {
                System.out.print(f.getName());
                System.out.print(": ");

                if (f instanceof TdbStructuredField)
                {
                    boolean bFirstSubfield = true;
                    for (String subfield : f.values())
                    {
                        if (bFirstSubfield)
                        {
                            bFirstSubfield = false;
                        }
                        else
                        {
                            System.out.print(", ");
                        }
                        System.out.print(subfield);
                    }
                    System.out.println();
                }
                else if (f instanceof TdbTextField)
                {
                    TdbTextField txtField = (TdbTextField)f;
                    System.out.println();
                    System.out.println(txtField.getText());
                }
            }

            System.out.println();
            if (++recordIdx >= maxRecords)
                break;
        }

        // Release server resources (including search sets) assoicated
        // with the TdbSearch object
        searchHandler.close();
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        try
        {
            SearchSetIteration app = new SearchSetIteration();

            Console stdin = new Console();
            String db = null, qry = null, refinement = null, fld = null;
            Vector<String> fetchFields = new Vector<String>();
            int maxRecords = 0;

            // Get a valid database name, and set the list of
            // retrieval fields from it
            while (db == null)
            {
                db = stdin.readln("Database", null);
            }

            app.loadDatabaseDesign(db);
            if (stdin.readint("Custom retrieval template? (1|0)", 1, true) != 0)
            {
                do
                {
                    fld = stdin.readln("Field",null);
                    if (fld != null)
                    {
                        if (app.isDatabaseHeadField(fld))
                            fetchFields.add(fld);
                        else
                        {
                            System.out.println("ERROR: No such field in the specified database.");
                        }
                    }
                }
                while (fetchFields.size() == 0 || fld != null);
            }
            

            // Prompt for a query
            while (qry == null)
            {
                qry = stdin.readln("Query (omit \"FIND\" keyword)", null);
            }

            // Prompt for an optional refinement query
            refinement = stdin.readln("Refinement query (omit \"FIND\" keyword)", null);

            // Prompt for max records to retrieve
            maxRecords = stdin.readint("Max records to retrieve (10)", 10, true);

            // Perform the retrieval
            app.search(qry, refinement, fetchFields, maxRecords);
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
    }
}
